#include "LibItsCam_TestSystem.hh"

#include "CAMCodec.hh"
#include "asn1/asn_application.h" // from asn1c

#include "loggers.hh"

int CAMCodec::encode (const CAM__PDU__Descriptions::CAM& cam, OCTETSTRING& data)
{
  loggers::get_instance().log(">>> CAMCodec::encode");
  BITSTRING b;
  int rc = asnCodec.encode(cam, b);
  if(rc){
    data = bit2oct(b);
    loggers::get_instance().log_msg("CAMCodec::encode: ", data);
  }
  return rc;
}

int CAMCodec::decode (const OCTETSTRING& data, CAM__PDU__Descriptions::CAM& cam, Params* params)
{
  loggers::get_instance().log_msg(">>> CAMCodec::decode: ", data);
  
  int rc = asnCodec.decode(oct2bit(data), cam);
  loggers::get_instance().log("CAMCodec::decode: ASN.1 codec returned %d", rc);
  if(rc) {
    loggers::get_instance().log_msg("CAMCodec::decode: ", cam);
  }
  return rc;
}

extern "C" {
  extern asn_TYPE_descriptor_t asn_DEF_CAM;
}

int CAMPDUCodec::encode (const CAM__PDU__Descriptions::CAM& p_cam, BITSTRING& p_data)
{
  return _encode(CAM__PDU__Descriptions::CAM_descr_, asn_DEF_CAM, p_cam, p_data);
}

int CAMPDUCodec::decode (const BITSTRING& p_data, CAM__PDU__Descriptions::CAM& p_cam)
{
  return _decode(CAM__PDU__Descriptions::CAM_descr_, asn_DEF_CAM, p_data, p_cam);
}
/*


extern "C" {
    extern asn_TYPE_descriptor_t asn_DEF_CAM;
    static int asn1c_collect_encoded_data(const void *buffer, size_t size, void *application_specific_key)
    {
      TTCN_Buffer * tb = (TTCN_Buffer *)application_specific_key;
      tb->put_s(size, (unsigned char *)buffer);
      return 0;
    }
}

int CAMPDUCodec::encode (const CAM__PDU__Descriptions::CAM& p_cam, BITSTRING& p_data)
{
  loggers::get_instance().log_msg(">>> CAMPDUCodec::encode: ", p_cam);
  
  int rc = -1;
  // Encode as BER
  // Encode message in BER (CER variant, but can be any)
  TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL, TTCN_EncDec::EB_WARNING);
  TTCN_Buffer buf;
  p_cam.encode(CAM__PDU__Descriptions::CAM_descr_, buf, TTCN_EncDec::CT_BER, BER_ENCODE_DER);
  loggers::get_instance().log_to_hexa("CAMPDUCodec::encode: BER encoding=", buf);

  // Decode BER
  void * ptr = NULL;
  asn_dec_rval_t rc_d;
  rc_d = asn_decode(NULL, ATS_BER, &asn_DEF_CAM, &ptr, buf.get_data(), buf.get_len());
  if(rc_d.code == RC_OK){
	// Encode as PER
    asn_enc_rval_t rc_e;
    buf.clear();
    rc_e = asn_encode(NULL, ATS_UNALIGNED_CANONICAL_PER, &asn_DEF_CAM, ptr,
			asn1c_collect_encoded_data, &buf);
    if(rc_e.encoded < 0) {
      loggers::get_instance().warning("CAMPDUCodec: PER encoding failed");
    }else{
      rc = rc_e.encoded * 8;
      p_data = p_data + oct2bit(OCTETSTRING(buf.get_len(), buf.get_data()));
    }
  } else {
    loggers::get_instance().warning("CAMPDUCodec: BER decoding failed");
  }
  if(ptr){
    ASN_STRUCT_FREE(asn_DEF_CAM, ptr);
  }
  return rc;
}

int CAMPDUCodec::decode (const BITSTRING& p_data, CAM__PDU__Descriptions::CAM& p_cam)
{
  loggers::get_instance().log(">>> CAMPDUCodec::decode");
  int rc = -1;
  void * ptr = NULL;
  // Decode UPER
  OCTETSTRING os = bit2oct(p_data);
  asn_dec_rval_t rc_d = asn_decode(NULL, ATS_UNALIGNED_BASIC_PER, &asn_DEF_CAM, &ptr, static_cast<const unsigned char*>(os), os.lengthof());
  if(rc_d.code == RC_OK) {
    // Encode BER
    TTCN_Buffer buf;
    asn_enc_rval_t rc_e = asn_encode(NULL, ATS_DER, &asn_DEF_CAM, ptr,
    		asn1c_collect_encoded_data, &buf);
    if (rc_e.encoded >= 0) {
      loggers::get_instance().log_to_hexa("CAMPDUCodec::decode: PER encoding=", buf);
      p_cam.decode(CAM__PDU__Descriptions::CAM_descr_, buf, TTCN_EncDec::CT_BER, BER_ACCEPT_ALL);
      rc = buf.get_len() * 8;
      loggers::get_instance().log("<<< CAMPDUCodec::decode: %d bits", rc);
    } else {
        loggers::get_instance().warning("CAMPDUCodec::decode: BER encoding failure");
    }
  } else {
    loggers::get_instance().warning("CAMPDUCodec::decode: PER");
  }
  if(ptr){
    ASN_STRUCT_FREE(asn_DEF_CAM, ptr);
  }
  return rc;
}
*/